@@ -13,6 +13,7 @@ import sys
 import time
 import bisect
 import hashlib
+import warnings
 
 try:
     from ansi2html import Ansi2HTMLConverter, style
@@ -157,14 +158,20 @@ class HTMLReport(object):
         def append_extra_html(self, extra, extra_index, test_index):
             href = None
             if extra.get('format') == extras.FORMAT_IMAGE:
-                if self.self_contained:
+                content = extra.get('content')
+                if content.startswith(('file', 'http')) or \
+                        os.path.isfile(content):
+                    if self.self_contained:
+                        warnings.warn('Self-contained HTML report '
+                                      'includes link to external '
+                                      'resource: {}'.format(content))
+                    html_div = html.a(html.img(src=content), href=content)
+                elif self.self_contained:
                     src = 'data:{0};base64,{1}'.format(
                         extra.get('mime_type'),
-                        extra.get('content'))
-                    self.additional_html.append(html.div(
-                        html.img(src=src), class_='image'))
+                        content)
+                    html_div = html.img(src=src)
                 else:
-                    content = extra.get('content')
                     if PY3:
                         content = b64decode(content.encode('utf-8'))
                     else:
@@ -172,9 +179,8 @@ class HTMLReport(object):
                     href = src = self.create_asset(
                         content, extra_index, test_index,
                         extra.get('extension'), 'wb')
-                    self.additional_html.append(html.div(
-                        html.a(html.img(src=src), href=href),
-                        class_='image'))
+                    html_div = html.a(html.img(src=src), href=href)
+                self.additional_html.append(html.div(html_div, class_='image'))
 
             elif extra.get('format') == extras.FORMAT_HTML:
                 self.additional_html.append(html.div(
@@ -192,6 +198,8 @@ class HTMLReport(object):
 
             elif extra.get('format') == extras.FORMAT_TEXT:
                 content = extra.get('content')
+                if isinstance(content, bytes):
+                    content = content.decode('utf-8')
                 if self.self_contained:
                     href = data_uri(content)
                 else:
@@ -213,9 +221,7 @@ class HTMLReport(object):
         def append_log_html(self, report, additional_html):
             log = html.div(class_='log')
             if report.longrepr:
-                for line in str(report.longrepr).splitlines():
-                    if not PY3:
-                        line = line.decode('utf-8')
+                for line in report.longreprtext.splitlines():
                     separator = line.startswith('_ ' * 10)
                     if separator:
                         log.append(line[:80])
